#
# shinGETsu - P2P anonymous BBS
#
# (c) FukTommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: Util.pm,v 1.15 2004/03/10 14:29:49 fuktommy Exp $
#

package Shingetsu::Util;
use Socket;
use Shingetsu::Config;
use Shingetsu::Extern;

#
# configuration
#
$PI   = 3.1415926535;
$lock = $Shingetsu::Config::lock;

sub lock(;$) {
	if ($^O ne "cygwin") {
		open LOCK, "> $lock" or die "$lock: open failed. $!";
		flock LOCK, 002;
	}
}

sub unlock() {
	if ($^O ne "cygwin") {
		close LOCK;
	}
}

#
# add a line to update list
# args: stamp, id, filename
#
sub addUpdate($$$) {
	my($stamp, $id, $filename) = @_;
	my $updatelist = $Shingetsu::Config::updatelist;
	my $saveupdate = $Shingetsu::Config::saveupdate;
	my @update = ();
	if (open IN, $updatelist) {
		@update = <IN>;
		close IN;
	}
	push @update, "$stamp<>$id<>$filename\n";
	local $_;
	my $now = time;
	Shingetsu::Util::lock();
	open OUT, "> $updatelist" or die "$updatelist: failed to open. $!";
	foreach (@update) {
		local @_ = split /<>/, $_;
		if ($now - $_[0] <= $saveupdate * 60) {
			print OUT;
		}
	}
	close OUT;
	Shingetsu::Util::unlock();
}

#
# nomal random (Box-Muller)
#
sub normal() {
	my($r1, $r2) = (rand, rand);
	return sqrt(-2 * log($r1)) * sin(2 * $PI * $r2);
}

#
# time with error
#
sub time() {
	my $now = CORE::time;
	my $error = int($Shingetsu::Config::timeError * normal());
	return $now + $error;
}

#
# localtime wrapper for Cygwin's bug
#
sub localtime(;@) {
	my $time;
	if (@_) {
		$time = $_[0];
	} else {
		$time = CORE::time();
	}
	if ((defined $ENV{TZ}) && ($ENV{TZ} ne "")) {
		return gmtime($time - substr($ENV{TZ}, 3) * 60*60);
	} else {
		return CORE::localtime($time);
	}
}

#
# MD5 check records
#
sub md5check(@) {
	my(@record) = @_;
	my @buf = ();
	local (@_, $_);
	foreach (@record) {
		@_ = split /<>/, $_, 3;
		if (@_ < 3) {
			next;
		} elsif ($_[1] eq Shingetsu::Extern::md5digest($_[2])) {
			push @buf, $_;
		}
	}
	return @buf;

}

#
# parse cache record
#
sub rec($) {
	local($_) = @_;
	chomp;
	@_ = split /<>/;
	my %rec = (stamp=> shift @_, id=> shift @_);
	foreach (@_) {
		my @buf = split /:/, $_, 2;
		next if (@buf < 2);
		$rec{$buf[0]} = $buf[1];
	}
	return %rec;
}

#
# wget
#
sub wget($) {
	my($uri) = @_;
	$uri =~ m|http://(.+):(\d+)/(.*)| or return ();
	my($host, $port, $path) = ($1, $2, "/$3");
	my $timeout = $Shingetsu::Config::timeout;
	my $bufsize = $Shingetsu::Config::bufsize;
	pipe IN, OUT;
	if (fork) {
		local $_;
		my @buf = ();
		local $SIG{CHLD} = "DEFAULT";
		close OUT;
		while (<IN>) {
			push @buf, $_;
		}
		wait;
		return @buf;
	} else {
		my($addr, $sockaddr);
		my $try = $Shingetsu::Config::retry;
		local $SIG{ALRM} = sub { close OUT;
					 die "$host:$port: connection timeout."; };
		close IN;
		START:
		alarm $timeout;
		unless ($addr = inet_aton $host) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host: faild to resolv. $!";
		}
		$sockaddr = pack_sockaddr_in($port, $addr);
		unless (socket SOCKET, PF_INET, SOCK_STREAM, 0) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "failed to open socket. $!";
		}
		unless (connect SOCKET, $sockaddr) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host:$port: fault to connect. $!";
		}
		select SOCKET; $| = 1;
		local $, = "\r\n";
		local $_;
		alarm $timeout;
		my $ver = $Shingetsu::Config::version;
		print "GET $path HTTP/1.0",
		      "User-Agent: shinGETsu/$ver",
		      "Host: $host:$port",
		      "Accept: text/plain",
		      "Accept-Encoding: gzip",
		      "Connection: close",
		      "\r\n";
		my $buf;
		while (read SOCKET, $buf, $bufsize) {
			print OUT $buf;
			alarm $timeout;
		}
		alarm 0;
		close SOCKET;
		close OUT;
		exit;
	}
}

1;
